/*****************************************************************************
 *
 * FILE:	login.c
 * DESCRIPTION:	login CGI to start a session
 * DATE:	Thu, May  6 2004
 * UPDATE:	Mon, Nov  8 2004
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: login.c,v 1.3 2004/11/09 14:56:39 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#include <cockatrice.h>
#include <catoblepas.h>
#include <sxml.h>
#include "session.h"
#include "wspasswd.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#define	WM_AUTH_NG	(-1)
#define	WM_AUTH_OK	(0)

#define	DEF_EXPIRES	(5*60)	/* 5 min. */

#define	PARAM_USER_ID	"user_id"
#define	PARAM_PASSWORD	"password"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static const char *	login(CGI *);
static int		authenticate(CGI *);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/
static const char *
login(cgi)
	register CGI *	cgi;
{
  if (cgi->session->start(SESSION_NAME) == 0) {
    const char *	sid;

    sid = cgi->session->generate_id2(cgi->param(PARAM_USER_ID),
				     cgi->param(PARAM_PASSWORD),
				     SESSION_NAME, DEF_EXPIRES);
    if (sid != NULL) {
      cgi->cookie->setName(SESSION_ID, sid);
      cgi->cookie->setMaxAge(SESSION_ID, DEF_EXPIRES);
      cgi->cookie->setPath(SESSION_ID, "/");
    }
    cgi->session->close(SESSION_NAME);

    return sid;
  }

  return NULL;
}

static int
authenticate(cgi)
	register CGI *	cgi;
{
  const char *	id;	/* user id */
  const char *	pass;	/* password */
  int		retval = WM_AUTH_NG;

  id   = cgi->param(PARAM_USER_ID);
  pass = cgi->param(PARAM_PASSWORD);
  if (id == NULL || pass == NULL) {
    return -1;
  }

  if (cgi->database->open(AUTH_DB_NAME) == 0) {
    char *	s;	/* quoted string */

    s = cgi->database->quote(id);
    if (s != NULL) {
      char *	q;	/* SQL query statement */

      asprintf(&q, "SELECT password FROM " AUTH_DB_NAME " WHERE name = %s;", s);
      if (q != NULL) {
	if (cgi->database->query(q) == 0) {
	  register int	n;
	  const char **	val;

	  n = cgi->database->fetch(&val);
	  if (n == 1) {
	    char *	p;

	    p = cgi->util->digest(pass, CC_DIGEST_MD5);
	    if (strcmp(val[0], p) == 0) {
	      retval = WM_AUTH_OK;
	    }
	  }
	}
	free((char *)q);
      }
      free((char *)s);
    }
    cgi->database->close();
  }

  return retval;
}


int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  CGI *	cgi;
  int	ret = -1;

#if	SESSION2
  cgi = newCGI(CC_MODULE_STANDARD|CC_MODULE_SESSION2|CC_MODULE_DATABASE);
#else
  cgi = newCGI(CC_MODULE_STANDARD|CC_MODULE_SESSION|CC_MODULE_DATABASE);
#endif	/* SESSION2 */
  if (cgi != NULL) {
    URI	uri;

    if (read_config(argv[0], &uri) == 0) {
      ret = 0;
#if	DEBUG
      fprintf(stderr, "DEBUG[login.cgi] %s, %s\n", uri.login, uri.startup);
      if (authenticate(cgi) == 0) {
	const char *	sid;

	sid = login(cgi);
	if (sid != NULL) {
	  char *	start;

	  asprintf(&start, "%s?%s=%s", uri.startup, SESSION_ID, sid);
	  if (start != NULL) {
	    cgi->redirect(start);
	    fprintf(stderr, "DEBUG[login.cgi] start session: %s\n", start);
	  }
	  else {
	    cgi->redirect(uri.startup);
	  }
	  free((char *)start);
	}
	else {
	  fprintf(stderr, "DEBUG[login.cgi] login failed\n");
	  cgi->redirect(uri.login);
	}
      }
      else {
	fprintf(stderr, "DEBUG[login.cgi] authentication failed\n");
	cgi->redirect(uri.login);
      }
#else
      if (authenticate(cgi) == 0 && login(cgi) != NULL) {
	cgi->redirect(uri.startup);
      }
      else {
	cgi->redirect(uri.login);
      }
#endif	/* DEBUG */
      if (uri.login   != NULL) { free((char *)uri.login); }
      if (uri.startup != NULL) { free((char *)uri.startup); }
    }
    else {
      fprintf(stderr, "Error: failed to read %s.conf\n", argv[0]);
    }
    cgi->done();
  }

  return ret;
}

