/*****************************************************************************
 *
 * FILE:	wspasswd.c
 * DESCRIPTION:	Wyvern: the front end program for mod_login_session.so
 * DATE:	Fri, May  7 2004
 * UPDATE:	Tue, Jul  6 2004
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *  
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *	documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: wspasswd.c,v 1.3 2004/11/09 14:56:39 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#if	HAVE_STDLIB_H
#include <stdlib.h>
#endif	/* HAVE_STDLIB_H */
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#if	HAVE_STRING_H
#include <string.h>
#endif	/* HAVE_STRING_H */
#if	HAVE_FCNTL_H
#include <fcntl.h>
#endif	/* HAVE_FCNTL_H */
#include <errno.h>
#include <sys/types.h>
#include <md5.h>
#include <sqlite.h>
#include "ccpath.h"
#include "wspasswd.h"

/*****************************************************************************
 *
 *	Copyright
 *
 *****************************************************************************/
const char	Program[]   = "wspasswd";
const char	Copyright[] =
	"Copyright (c) 2004 Kouichi ABE (WALL), All rights reserved.";

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
#ifndef	MD5_DIGEST_LENGTH
#define	MD5_DIGEST_LENGTH	(16)
#endif	/* MD5_DIGEST_LENGTH */

#define	EOL	'\0'
#define CR	'\015'
#define LF	'\012'

#define	USERNAME_SIZE		(16)
#define	PASSWORD_SIZE		(24)

#define	SQLITE_DB_TABLE	\
	"CREATE TABLE " AUTH_DB_NAME "(" \
	"name VARCHAR(16) UNIQUE," \
	"password VARCHAR(24)" \
	");"
#define	SQLITE_DB_TABLE0	\
	"CREATE TABLE " AUTH_DB_NAME "(" \
	"name VARCHAR(" USERNAME_SIZE ") UNIQUE," \
	"password VARCHAR(" PASSWORD_SIZE ")" \
	");"

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static int	submit(const char *, const char *, const char *);
static char *	to_MD5(const char *);
static void	usage(void);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

static int
submit(username, password, database)
	const char *	username;
	const char *	password;
	const char *	database;
{
  char *	path;

  asprintf(&path, "%s/%s.%s", SQLITE_DB_DIR, database, SQLITE_DB_SUFFIX);
  if (path) {
    sqlite *	sdb;
    char *	errMsg;

    sdb = sqlite_open(path, 0, &errMsg);	free((char *)path);
    if (sdb) {
      int	n;

again:
      /* insert data */
      n = sqlite_exec_printf(sdb,
			     "INSERT INTO " AUTH_DB_NAME " VALUES('%s', '%s')",
			     NULL, NULL, &errMsg, username, to_MD5(password));
      if (n != SQLITE_OK) {
	if (errMsg) {
	  fprintf(stderr, "Error: %s\nretrying...\n", errMsg);
	  sqlite_freemem(errMsg);
	}
	/* create table */
	n = sqlite_exec(sdb, SQLITE_DB_TABLE, NULL, NULL, &errMsg);
	if (n == SQLITE_OK) {
	  goto again;
	}
	if (errMsg) {
	  fprintf(stderr, "Error: %s ... give up!\n", errMsg);
	  sqlite_freemem(errMsg);
	}
      }
      else {
	fprintf(stderr, "successful\n");
      }
      sqlite_close(sdb);

      return n;
    }
    if (errMsg) {
      fprintf(stderr, "Error: %s\n", errMsg);
      sqlite_freemem(errMsg);
    }
  }

  return -1;
}

/*
 * FUNCTION NAME:	to_MD5
 * DESCRIPTION:		make a new password
 * ARGUMENTS:		password (plain text)
 * RETURN VALUE:	a crypted password
 */
static char *
to_MD5(text)
	const char *	text;
{
  MD5_CTX		md5_ctx;
  static unsigned char	md5_hex[MD5_DIGEST_LENGTH*2+1];

  memset(md5_hex, 0, sizeof(md5_hex));
  MD5Init(&md5_ctx);
  MD5Update(&md5_ctx, text, strlen(text));
  MD5End(&md5_ctx, md5_hex);

  return md5_hex;
}

/*
 * FUNCTION NAME:	usage
 * DESCRIPTION:		print usage
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
static void
usage(void)
{
  fprintf(stderr,
	  "%s\n"
	  "usage: %s <username> <password> <database>\n",
	  Copyright, Program);
  exit(64);
}


/*
 *	MAIN FUNCTION
 */
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
#define	ARGS_USERNAME	(1)
#define	ARGS_PASSWORD	(2)
#define	ARGS_DATABASE	(3)
  if (argc != 4) {
    usage();
  }

  if (strlen(argv[ARGS_USERNAME]) > USERNAME_SIZE) {
    fprintf(stderr, "Error: username length is over %d.\n", USERNAME_SIZE);
    exit(1);
  }
  if (strlen(argv[ARGS_PASSWORD]) > PASSWORD_SIZE) {
    fprintf(stderr, "Error: password length is over %d.\n", PASSWORD_SIZE);
    exit(1);
  }

  submit(argv[ARGS_USERNAME], argv[ARGS_PASSWORD], argv[ARGS_DATABASE]);

  return 0;
}

