/*****************************************************************************
 *
 * FILE:	module.h
 * DESCRIPTION:	Cockatrice: module header file
 * DATE:	Tue, Apr 20 2004
 * UPDATED:	Mon, Aug 20 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2004-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2004-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: module.h,v 1.9 2007/08/20 15:33:00 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_MODULE_H
#define	_MODULE_H

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/

#ifndef	CC_MODULE_MAGIC_NUMBER_MAJOR
#define	CC_MODULE_MAGIC_NUMBER_MAJOR	0x20070818
#endif	/* CC_MODULE_MAGIC_NUMBER_MAJOR */
#define	CC_MODULE_MAGIC_NUMBER_MINOR	0	/* 0...n */

/* Module structure */
typedef struct {
  const int	version;		/* API version */
  const char *	name;			/* module name */
  const char *	info;			/* description such as version etc.. */
  void *	handle;			/* module handle */
  void		(*init)(void);		/* called when it's loaded */
  void		(*finish)(void);	/* called when it's unloaded */
#define	CC_MODULE_DLSYM_NAME	"ccModuleInfo"
  void		(*registration)(void);
} ccModule;

#define	CM_MODULE_PATH		COCKATRICE_MODULE_PATH "/"
#define	CM_FILE_FILE		CM_MODULE_PATH "file.so"
#define	CM_ENV_FILE		CM_MODULE_PATH "env.so"
#define	CM_DATE_FILE		CM_MODULE_PATH "date.so"
#define	CM_CHECK_FILE		CM_MODULE_PATH "check.so"
#define	CM_SESSION_FILE		CM_MODULE_PATH "session.so"
#define	CM_SQLITE3_FILE		CM_MODULE_PATH "sqlite3.so"
#define	CM_PGSQL8_FILE		CM_MODULE_PATH "pgsql8.so"

/*****************************************************************************
 *
 *	Global functions declaration
 *
 *****************************************************************************/
extern ccModule *	ccModuleInfo(void);
/* cockatrice.c */
extern void		cc_hook_response_header(void (*)(const char *));
extern void		cc_hook_redirect_uri(void (*)(const char *));
extern void		cc_hook_module_uri(const cmURI *);
extern void		cc_hook_module_file(const cmFile *);
extern void		cc_hook_module_util(const cmUtil *);
extern void		cc_hook_module_cookie(const cmCookie *);
extern void		cc_hook_module_upload(const cmUpload *);
extern void		cc_hook_module_env(const cmEnv *);
extern void		cc_hook_module_session(const cmSession *);
extern void		cc_hook_module_database(const cmDatabase *);
extern void		cc_hook_module_date(const cmDate *);
extern void		cc_hook_module_check(const cmCheck *);
/* module.c */
extern int		cc_load_module(const char *);
extern int		cc_unload_module(const char *);
extern int		cc_load_modules(int);
extern void		cc_initialize_modules(void);
extern void		cc_remove_modules(void);

#endif	/* _MODULE_H */
